package com.mortals.xhx.module.setup.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RuntimeUtil;
import cn.hutool.core.util.ZipUtil;
import cn.hutool.db.DbUtil;
import cn.hutool.system.OsInfo;
import cn.hutool.system.SystemUtil;
import com.mortals.framework.common.Rest;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.ProductDisEnum;
import com.mortals.xhx.common.code.ProjectStatusEnum;
import com.mortals.xhx.common.code.ProjectTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.ZipUtils;
import com.mortals.xhx.module.setup.mode.DbSetupEntity;
import com.mortals.xhx.module.setup.mode.SiteEntity;
import com.mortals.xhx.module.setup.service.SetupDbService;
import org.springframework.beans.BeanUtils;

import java.io.*;
import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.setup.dao.SetupProjectDao;
import com.mortals.xhx.module.setup.model.SetupProjectEntity;
import com.mortals.xhx.module.setup.service.SetupProjectService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.sql.DataSource;

/**
 * SetupProjectService
 * 项目工程信息 service实现
 *
 * @author zxfei
 * @date 2024-10-21
 */
@Service("setupProjectService")
@Slf4j
public class SetupProjectServiceImpl extends AbstractCRUDServiceImpl<SetupProjectDao, SetupProjectEntity, Long> implements SetupProjectService {

    @Autowired
    private UploadService uploadService;

    @Autowired
    private SetupDbService setupDbService;

    @Value("${project.publishPath:'/home/publish'}")
    private String publishPath;


    @Override
    public Rest<String> distribute(SetupProjectEntity setupProjectEntity, Context context) {
        //TODO: 业务逻辑 判断是后端还是前端

        if (ProjectTypeEnum.后端.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 后端逻辑
            //TODO: 判断打包文件是否存在，如果不存在 则使用默认的资源文件夹的包文件
            if (!ObjectUtils.isEmpty(setupProjectEntity.getSourceProject())) {
                //判断文件是否存在
                String targetFilePath = uploadService.getFilePath(setupProjectEntity.getSourceProject());
                if (FileUtil.isEmpty(new File(targetFilePath))) {
                    throw new AppException("部署文件不存在！");
                }
                //根据不同的文件类型进行解压
                if (FileUtil.getSuffix(setupProjectEntity.getSourceProject()).equals("zip")) {

                } else if (FileUtil.getSuffix(setupProjectEntity.getSourceProject()).equals("tar.gz")) {

                }

            } else {
                //读取本工程的资源文件 检测压缩包类型
                //String sourcePath = "/project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue() + ".tar.gz";
                //InputStream inputStream = this.getClass().getResourceAsStream(sourcePath);
                String sourcePath = this.publishPath+"/temp/project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue() + ".tar.gz";

                File file = new File(sourcePath);
                log.info("文件存在：{}", file.exists());

                if(! file.exists()){
                    throw new AppException("请上传项目工程文件！");
                }

                InputStream inputStream = FileUtil.getInputStream(file);

                ZipUtils.unGzip(inputStream, setupProjectEntity.getProjectPath());

                String publicPath = setupProjectEntity.getProjectPath() + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue();

                //导入数据库业务
                DbSetupEntity dbSetupEntity = new DbSetupEntity();
                dbSetupEntity.setDbPort(setupProjectEntity.getDbPort());
                dbSetupEntity.setDbHost(setupProjectEntity.getDbHost());
                dbSetupEntity.setDbName(setupProjectEntity.getDbName());
                dbSetupEntity.setUserName(setupProjectEntity.getDbUser());
                dbSetupEntity.setPassword(setupProjectEntity.getDbPassword());
                dbSetupEntity.setDbFilePath(publicPath + "/db/base.sql");
                log.info("数据库db路径！{}", dbSetupEntity.getDbFilePath());
                Rest<String> dbRest = setupDbService.initDb(dbSetupEntity);
                if (YesNoEnum.YES.getValue() != dbRest.getCode()) {
                    log.info("数据库初始化成功！");
                }

                setupProjectEntity.setProjectStatus(ProjectStatusEnum.已部署.getValue());
                this.update(setupProjectEntity);

                //判断当前系统类型，执行不同shell 或者 bat脚本
                if (SystemUtil.getOsInfo().isLinux()) {
                    //todo 执行部署脚本
                    //后端服务，创建service  启动服务
                    //RuntimeUtil.exec()
                    log.info("执行deploy shell 脚本 或者构造文件,{}",publicPath+"/bin/");
                     callScript("project_deploy.sh","4",publicPath+"/bin/");
                } else if (SystemUtil.getOsInfo().isWindows()) {
                    //todo
                }

            }


        } else if (ProjectTypeEnum.前端.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 前端逻辑
        } else if (ProjectTypeEnum.资源.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 资源逻辑
        }


        //判断打包文件是否存在，


        return Rest.ok();
    }

    @Override
    public Rest<String> appDeploy(SetupProjectEntity setupProjectEntity, Context context) {

        String sourcePath = "./../project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/app.tar.gz";

        //创建目录
        File appDir = new File(setupProjectEntity.getProjectPath() + "app/");
        File appSiteDir = new File(setupProjectEntity.getProjectPath() + "app/" + setupProjectEntity.getSiteCode() + "/");
        if (!FileUtil.exist(appDir)) {
            FileUtil.mkdir(appDir);
        }

        if (!FileUtil.exist(appSiteDir)) {
            FileUtil.mkdir(appSiteDir);
        }

        File file = new File(sourcePath);
        log.info("文件存在：{}", file.exists());
        InputStream inputStream = FileUtil.getInputStream(file);

        ZipUtils.unGzip(inputStream, appSiteDir.getPath());

        return Rest.ok("应用部署成功！");
    }


    // @Override
    public Rest<String> updateBaseSystem(SiteEntity siteEntity) {

        //获取数据库连接 如果基础服务不存在 则提示
        DbSetupEntity dbSetupEntity = new DbSetupEntity();

        dbSetupEntity.setDbName(siteEntity.getDbName());
        dbSetupEntity.setDbHost(siteEntity.getDbHost());
        dbSetupEntity.setDbPort(siteEntity.getDbPort());
        dbSetupEntity.setUserName(siteEntity.getUserName());
        dbSetupEntity.setPassword(siteEntity.getPassword());


        Rest<String> connect = setupDbService.connect(dbSetupEntity);
        if (YesNoEnum.YES.getValue() != connect.getCode()) {
            return Rest.fail("数据库连接失败");
        }

        Rest<DataSource> rest = setupDbService.getDataSource(dbSetupEntity);
        DataSource dataSource = rest.getData();
        Connection connection = null;
        try {
            connection = dataSource.getConnection();
            //updateSite(connection, dbSetupEntity.getSiteEntity()

        } catch (SQLException e) {
            throw new RuntimeException(e.getMessage());
        }


        return null;
    }


    public boolean updateSite(Connection connection, SiteEntity site) throws AppException {
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        try {
            // 构建Sql,参数使用占位符代替
            // 手动拼接SQL
            // String sql = "update mortals_sys_site set ";
            //todo
            String sql = "";
            List<Object> paramList = new ArrayList<Object>();
            //如果某个属性为null，则跳过，不更新此字段
            if (site.getSiteName() != null) {
                sql += "siteName = ?,";
                paramList.add(site.getSiteName());
            }

            // 去除多余的逗号
            sql = sql.substring(0, sql.length() - 1);
            sql += " where id = ?";
            // 添加id到参数列表中
            paramList.add(site.getId());
            // 创建PreparedStatement对象
            statement = connection.prepareStatement(sql);
            // 设置sql中占位符对应的值
            for (int i = 0; i < paramList.size(); i++) {
                statement.setObject(i + 1, paramList.get(i));
            }
            int result = statement.executeUpdate();
            // 如果影响的行数大于0，表示此用户更新成功
            if (result > 0) {
                return true;
            }
        } catch (Exception e) {
            log.error("更新站点信息失败", e);
            throw new AppException(e.getMessage());
        } finally {
            DbUtil.close(resultSet, statement, connection);

        }
        return false;
    }


    private void callScript(String script, String args, String... workspace) {
        try {
            String cmd = "sh " + script + " " + args;
//        	String[] cmd = {"sh", script, "4"};
            File dir = null;
            if (workspace[0] != null) {
                dir = new File(workspace[0]);
                System.out.println(workspace[0]);
            }
            String[] evnp = {"val=2", "call=Bash Shell"};
           // Process process = Runtime.getRuntime().exec("su");
            Process     process = Runtime.getRuntime().exec(cmd, evnp, dir);
//            process = Runtime.getRuntime().exec(cmd);
            BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = "";
            while ((line = input.readLine()) != null) {
                log.info(line);
            }
            input.close();
        } catch (Exception e) {
            log.error("执行脚本失败", e);
        }
    }


    @Override
    public Rest<String> unProjectZip(MultipartFile file, Context context) {

        String originalFilename = file.getOriginalFilename();
        log.info("originalFilename:{}", originalFilename);

        //解压目录，为当前工程目录 /home/publish/setup-manager/project/
        String projectPath = this.publishPath + "/temp/project/";

        boolean exist = FileUtil.exist(projectPath);
        if (!exist) {
            FileUtil.mkdir(new File(projectPath));
        }
        try {
            if (originalFilename.contains(".zip")) {
                ZipUtil.unzip(file.getInputStream(), new File(projectPath), Charset.defaultCharset());
            } else if (originalFilename.contains(".tar.gz")) {
                ZipUtils.unGzip(file.getInputStream(), projectPath);
            }
        } catch (Exception e) {
            log.error("解压文件失败", e);
            throw new AppException(e.getMessage());

        }
        return Rest.ok();
    }


    public static void main(String[] args) {
        System.out.println(FileUtil.getSuffix("test.tar.gz"));

        //CallShell call = new CallShell();
        //call.callScript("test.sh", "4", "/root/experiment/");
    }
}