package com.mortals.xhx.module.setup.web;

import cn.hutool.core.io.FileUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.ProjectStatusEnum;
import com.mortals.xhx.common.code.ProjectTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.setup.mode.DbSetupEntity;
import com.mortals.xhx.module.setup.mode.ProjectSetupEntity;
import com.mortals.xhx.module.setup.mode.SiteEntity;
import com.mortals.xhx.module.setup.model.SetupProjectEntity;
import com.mortals.xhx.module.setup.service.SetupDbService;
import com.mortals.xhx.module.setup.service.SetupProjectService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 项目工程信息
 *
 * @author zxfei
 * @date 2024-10-21
 */
@RestController
@RequestMapping("setup/project")
public class SetupProjectController extends BaseCRUDJsonBodyMappingController<SetupProjectService, SetupProjectEntity, Long> {


    @Value("${project.nacosUrl}")
    private String nacosUrl;

    @Autowired
    private SetupDbService setupDbService;


    public SetupProjectController() {
        super.setModuleDesc("项目工程信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "projectType", ProjectTypeEnum.getEnumMap());
        this.addDict(model, "projectStatus", ProjectStatusEnum.getEnumMap());
        super.init(model, context);
    }

    @Override
    protected void doListBefore(SetupProjectEntity query, Map<String, Object> model, Context context) throws AppException {
        query.setSize(50);
        query.setOrderColList(Arrays.asList(new OrderCol("orderNum")));


        super.doListBefore(query, model, context);
    }


    @Override
    protected void doListAfter(SetupProjectEntity query, List<SetupProjectEntity> list, Context context) throws AppException {
        //todo 更新项目状态 通过shell cmd 命令等 查询项目状态

        super.doListAfter(query, list, context);
    }

    /**
     * 部署
     */
    @ApiOperation(value = "项目部署", notes = "部署")
    @PostMapping(value = "distribute")
    public Rest<Void> distribute(@RequestBody SetupProjectEntity setupProjectEntity) {
        String busiDesc = this.getModuleDesc() + "设备模块部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            this.service.distribute(setupProjectEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("项目部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    @PostMapping("/dbConnect")
    @UnAuth
    public String connect(@RequestBody SetupProjectEntity setupProjectEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(setupProjectEntity.getDbName())) {
                throw new AppException("请选择产品数据库名称");
            }

            //导入数据库业务
            DbSetupEntity dbSetupEntity = new DbSetupEntity();
            dbSetupEntity.setDbPort(setupProjectEntity.getDbPort());
            dbSetupEntity.setDbHost(setupProjectEntity.getDbHost());
            dbSetupEntity.setDbName(setupProjectEntity.getDbName());
            dbSetupEntity.setUserName(setupProjectEntity.getDbUser());
            dbSetupEntity.setPassword(setupProjectEntity.getDbPassword());

            Rest<String> rest = setupDbService.connect(dbSetupEntity);
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "连接数据库成功!");
            } else {
                throw new AppException(rest.getMsg());
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("链接数据库异常", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }

/*
    @PostMapping("/init")
    @UnAuth
    public String initDb(@RequestBody ResourceSetupEntity resourceSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(resourceSetupEntity.getResourceFilePath())) {
                throw new AppException("请上传资源文件");
            }
            String targetFilePath = uploadService.getFilePath(resourceSetupEntity.getResourceFilePath());
            if (FileUtil.isEmpty(new File(targetFilePath))) {
                throw new AppException("部署文件不存在！");
            }

            if (!FileUtil.getSuffix(resourceSetupEntity.getResourceFilePath()).equals("zip")) {
                throw new AppException("部署只支持zip文件！");
            }

            if (!FileUtil.isDirectory(filePath)) {
                //部署路径是否存在 如果不存在 创建目录，
                FileUtil.mkdir(filePath);
            }

            String fileEncode = "UTF-8";
            try {
                fileEncode = EncodeUtil.getEncode(targetFilePath, true);
            } catch (Exception e) {
                log.error("异常", e);
            }
            ZipUtil.unzip(targetFilePath, filePath, Charset.forName(fileEncode));

            ret.put(KEY_RESULT_MSG, "初始化资源文件成功!");

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("导入资源文件失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }*/


    /**
     * 更新基础服务子区域
     *
     * @return
     */
    @PostMapping("/area/update")
    @UnAuth
    public String areaUpdate(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        if (ObjectUtils.isEmpty(siteEntity.getAreaCode())) throw new AppException("请输入区域编码");

        try {
            HashMap<String, String> params = new HashMap<>();
            params.put("areaCode", siteEntity.getAreaCode());

            String resp = HttpUtil.post("http://127.0.0.1:11078/base/area/genSubAreaByAreaName", JSON.toJSONString(params));
            Rest rest = JSON.parseObject(resp, Rest.class);

            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "子区域更新成功");
                SetupProjectEntity setupProjectEntity = new SetupProjectEntity();
                setupProjectEntity.setId(siteEntity.getId());
                setupProjectEntity.setAreaCode(siteEntity.getAreaCode());
                this.service.update(setupProjectEntity);
            }

            ret.put(KEY_RESULT_MSG, "子区域更新失败");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("子区域更新失败失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 创建站点
     *
     * @return
     */
    @PostMapping("/site/add")
    @UnAuth
    public String updateSite(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        if (ObjectUtils.isEmpty(siteEntity.getSiteName())) throw new AppException("请输入站点名称");

        SetupProjectEntity setupProject = this.service.get(siteEntity.getId());
        if (ObjectUtils.isEmpty(setupProject)) throw new AppException("请先创建项目");

        if (ObjectUtils.isEmpty(setupProject.getAreaCode())) throw new AppException("请先创建区域");

        try {

            String resp = HttpUtil.post("http://127.0.0.1:11078/base/site/api/add", JSON.toJSONString(siteEntity));
            Rest rest = JSON.parseObject(resp, Rest.class);
            log.info(JSON.toJSONString(rest));
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "站点添加成功!");
                SetupProjectEntity setupProjectEntity = new SetupProjectEntity();
                setupProjectEntity.setId(siteEntity.getId());
                setupProjectEntity.setSiteName(siteEntity.getSiteName());
                // setupProjectEntity.setSiteCode();
                //setupProjectEntity.setSiteId();
                this.service.update(setupProjectEntity);
            }
            //   ret.put(KEY_RESULT_DATA, resp);
            ret.put(KEY_RESULT_MSG, "站点添加成功！");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("站点添加失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 自助服务应用部署
     *
     * @return
     */
    @PostMapping("/app/deploy")
    @UnAuth
    public String appDeploy(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        if (ObjectUtils.isEmpty(siteEntity.getSiteCode())) throw new AppException("请先创建站点后再部署自助服务应用！");

        try {
            //todo 在/home/publish/app/ 目录下 创建自助服务应用

            ret.put(KEY_RESULT_MSG, "子区域更新失败");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("查询服务状态失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /* *//**
     * 服务列表查询
     *
     * @param projectSetupEntity
     * @return
     *//*
    @PostMapping("/list")
    @UnAuth
    public String list(@RequestBody ProjectSetupEntity projectSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            // curl -X GET 'http://127.0.0.1:8848/nacos/v1/ns/catalog/services?pageNo=1&pageSize=100&namespaceId=smart-gov'
            String resp = HttpUtil.get(nacosUrl + "/v1/ns/catalog/services?pageNo=1&pageSize=100&namespaceId=smart-gov");

            ret.put(KEY_RESULT_DATA, resp);
            ret.put(KEY_RESULT_MSG, "项目资源部署成功");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("查询服务状态失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }
*/

    /**
     * 服务实例详细查询
     *
     * @param projectSetupEntity
     * @return
     */
    @PostMapping("/instances")
    @UnAuth
    public String instances(@RequestBody ProjectSetupEntity projectSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            // curl -X GET 'http://127.0.0.1:8848/nacos/v1/ns/catalog/instances?serviceName=base-manager&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov'
            String resp = HttpUtil.get(nacosUrl + "/v1/ns/catalog/instances?serviceName=" + projectSetupEntity.getProjectValue() + "&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov");
            ret.put(KEY_RESULT_DATA, resp);
            ret.put(KEY_RESULT_MSG, "服务实例详细查询成功");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("服务实例详细查询失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 导入文档生成代码
     *
     * @param file
     * @return
     */
    @PostMapping("/importProjectData")
    @UnAuth
    public String importDoc(MultipartFile file) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        Context context = getContext();
        String busiDesc = "导入" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {

            //解压
            this.service.unProjectZip(file, context);

            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "导入成功！");
        return ret.toJSONString();
    }


    public static void main(String[] args) {

        System.out.println(FileUtil.isDirectory("E://pic"));

    }

}